package com.bfm.bpm.hibernate;

import java.io.InputStream;
import java.io.Serializable;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.Arrays;

import org.hibernate.Hibernate;
import org.hibernate.HibernateException;
import org.hibernate.lob.SerializableBlob;
import org.hibernate.usertype.UserType;
import org.jbpm.pvm.internal.util.IoUtil;

public class SybaseBlobType implements UserType {
    public int[] sqlTypes() {
        return new int[]{Types.LONGVARBINARY};
    }

    public Class returnedClass() {
        return byte[].class;
    }

    public boolean equals(Object x, Object y) {
        byte[] b1 = readBytes(x);
        byte[] b2 = readBytes(y);
        return (x == y) || (x != null && y != null && Arrays.equals(b1, b2));
    }

    public int hashCode(Object o) throws HibernateException {
        return o.hashCode();
    }

    public Object nullSafeGet(ResultSet rs, String[] names, Object owner) throws HibernateException, SQLException {
        InputStream inputStream = rs.getBinaryStream(names[0]);

        if (inputStream == null) {
            return null;
        }

        byte[] bytes = IoUtil.readBytes(inputStream);

        return new SerializableBlob(Hibernate.createBlob(bytes));
    }

    public void nullSafeSet(PreparedStatement st, Object value, int index) throws HibernateException, SQLException {
        SerializableBlob blob = (SerializableBlob) value;
        if (blob != null) {
            st.setBytes(index, readBytes(value));
        }
        else {
            st.setNull(index, Types.LONGVARBINARY);
        }
    }

    public Object deepCopy(Object value) {
        if (value == null) {
            return null;
        }

        byte[] bytes = readBytes(value);
        byte[] result = new byte[bytes.length];
        System.arraycopy(bytes, 0, result, 0, bytes.length);

        return new SerializableBlob(Hibernate.createBlob(result));
    }

    private byte[] readBytes(Object value) {
        if (value == null) {
            return null;
        }

        try {
            return IoUtil.readBytes(((SerializableBlob) value).getBinaryStream());
        } catch (SQLException e) {
            throw new HibernateException(e);
        }
    }

    public boolean isMutable() {
        return false;
    }

    public Serializable disassemble(Object o) throws HibernateException {
        return null;
    }

    public Object assemble(Serializable serializable, Object o) throws HibernateException {
        return null;
    }

    public Object replace(Object o, Object o1, Object o2) throws HibernateException {
        return null;
    }

}
